@@ -18,10 +18,9 @@
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #   GNU General Public License for more details.
 #
-#   You should have received a copy of the GNU General Public
-#   License along with this program; if not, write to the Free
-#   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-#   MA 02111-1307 USA
+#   You should have received a copy of the GNU General Public License
+#   along with this program; if not, write to the Free Software
+#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 #
 
 import re
@@ -205,7 +204,7 @@ class Changes:
         return self._numeric_urgency
 
     @property
-    def entires(self):
+    def entries(self):
         return self._entries
 
     @property
@@ -239,13 +238,15 @@ class ChangelogParser:
     def __init__(self, changes):
         self._changes = changes
 
-    def parse(self, fd, since_version):
+    def parse(self, fd, since_version, latest):
         '''Parse changelog or news from the given file descriptor.
         If since_version is specified, only save entries later
-        than the specified version.'''
+        than the specified version.
+        If latest is specified, only the latest N versions.'''
         ancient = False
         entry = None
         is_debian_changelog = False
+        latest = int(latest) if latest else None
 
         for line in fd.readlines():
             line = line.decode('utf-8', 'replace')
@@ -268,6 +269,9 @@ class ChangelogParser:
                                                                  since_version) <= 0:
                         break
                     self._save_entry(entry)
+                    if latest is not None and latest <= len(self._changes.entries):
+                        entry = None
+                        break
                     entry = ChangelogEntry(line, version, match.group('urgency'),
                                            self._binnmu_marker in line)
 
@@ -296,10 +300,11 @@ class ChangelogParser:
             self._changes.save_entry(entry)
 
 class ChangelogsReader:
-    def __init__(self, source_pkg_name, rootdir, since_version, reverse):
+    def __init__(self, source_pkg_name, rootdir, since_version, latest, reverse):
         self._source_pkg_name = source_pkg_name
         self._rootdir = rootdir
         self._since_version = since_version
+        self._latest = latest
         self._reverse = reverse
 
     def read_changelogs(self, filenames, binnmus_filenames):
@@ -313,7 +318,7 @@ class ChangelogsReader:
             result = reduce(find_first, binnmus_filenames, False) or result
 
         if not result:
-            return None # none of the files was a valid debian changelog file
+            return None # none of the files was a valid Debian changelog file
 
         if self._reverse:
             changes.reverse()
@@ -326,7 +331,7 @@ class ChangelogsReader:
             return False
 
         with fd:
-            return ChangelogParser(changes).parse(fd, self._since_version)
+            return ChangelogParser(changes).parse(fd, self._since_version, self._latest)
 
     def _open_changelog_file(self, filename):
         filenames = glob.glob(filename)
@@ -358,11 +363,12 @@ class Package:
         self.Version = pkgdata.version()
         self.arch = pkgdata.Architecture
 
-    def extract_changes(self, which, since_version, reverse):
+    def extract_changes(self, which, since_version, latest, reverse):
         '''Extract changelog and binnmu entries, news or both from the package.
 
         Unpacks changelog or news files from the binary package, and parses them.
         If since_version is specified, only return entries later than the specified version.
+        If latest is specified, only the latest N versions.
         Returns a tuple of sequences of Changes objects.'''
 
         news_filenames = self._changelog_variations('NEWS.Debian')
@@ -380,7 +386,7 @@ class Package:
 
         tempdir = self._extract_contents(filenames)
         try:
-            reader = ChangelogsReader(self.source, tempdir, since_version, reverse)
+            reader = ChangelogsReader(self.source, tempdir, since_version, latest, reverse)
 
             news = reader.read_changelogs(news_filenames, None)
 
@@ -392,11 +398,12 @@ class Package:
 
         return (news, changelog)
 
-    def extract_changes_via_apt(self, since_version, reverse):
+    def extract_changes_via_apt(self, since_version, latest, reverse):
         '''Run apt-get changelog and parse the downloaded changelog.
 
         Retrieve changelog using the "apt-get changelog" command, and parse it.
         If since_version is specified, only return entries later than the specified version.
+        If latest is specified, only the latest N versions.
         Returns a single sequence of Changes objects or None on downloading or parsing failure.'''
 
         # Retrieve changelog file and save it in a temporary directory
@@ -423,7 +430,7 @@ class Package:
                               'errmsg': str(ex)})
 
         else:
-            return ChangelogsReader(self.source, '', since_version,
+            return ChangelogsReader(self.source, '', since_version, latest,
                                     reverse).read_changelogs([changelog_file], None)
 
         finally:
